__author__  = '''Simples Consultoria'''
__docformat__ = 'plaintext'

# Python imports
import StringIO
from cStringIO import StringIO
import string

from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ManagePortal
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.Archetypes import listTypes

# Config

def install(self):
    out=StringIO()
    
    setup_tool = getToolByName(self, 'portal_setup')
    
    setup_tool.runAllImportStepsFromProfile("profile-Products.BrFieldsAndWidgets:default", purge_old=False)

    out.write('Installation completed.\n')
    return out.getvalue()

def uninstall(self):
    out=StringIO()
    return out.getvalue()




from unittest import TestSuite, makeSuite
from Products.Archetypes.tests.atsitetestcase import ATSiteTestCase


class TestValidation(ATSiteTestCase):

    def test_cpf_registration(self):
        from Products.validation import validation
        v = validation.validatorFor('isCPF')
        self.failUnlessEqual(v('81057807559'), 1)
        self.failIfEqual(v('8105780755x'), 1)

    def test_cnpj_registration(self):
        from Products.validation import validation
        v = validation.validatorFor('isCNPJ')
        self.failUnlessEqual(v('90.656.353/0001-37'), 1)
        self.failIfEqual(v('90.656.353/0001-36'), 1)

    def test_cep_registration(self):
        from Products.validation import validation
        v = validation.validatorFor('isCEP')
        self.failUnlessEqual(v('05417-010'), 1)
        self.failIfEqual(v('05417'), 1)
        
    def test_phone_registration(self):
        from Products.validation import validation
        v = validation.validatorFor('isBrPhone')
        self.failUnlessEqual(v('(11)38982121'), 1)
        self.failIfEqual(v('113898-212121'), 1)

def test_suite():
    suite = TestSuite()
    
    suite.addTest(makeSuite(TestValidation))

    return suite


from Products.CMFCore import utils
from Products.CMFCore.DirectoryView import registerDirectory
from Products.Archetypes.public import process_types, listTypes
from Products.Archetypes.atapi import *
from Products.validation.config import validation

from config import *

registerDirectory(SKINS_DIR, GLOBALS)

from zope.i18nmessageid import MessageFactory as BaseMessageFactory
MessageFactory = BaseMessageFactory('Products.BrFieldsAndWidgets')

from Products.BrFieldsAndWidgets import validators
from Products.BrFieldsAndWidgets.content import BrFieldsAndWidgetsDemo


def initialize(context):
    
    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)
    utils.ContentInit(
        PROJECTNAME + ' Content',
        content_types      = content_types,
        permission         = ADD_CONTENT_PERMISSION,
        extra_constructors = constructors,
        fti                = ftis,
        ).initialize(context)


PROJECTNAME = "BrFieldsAndWidgets"
SKINS_DIR = 'skins'
GLOBALS = globals()

try:
    from Products.CMFCore.permissions import AddPortalContent
except:
    from Products.CMFCore.CMFCorePermissions import AddPortalContent

ADD_CONTENT_PERMISSION = AddPortalContent



__author__  = '''Simples Consultoria'''
__docformat__ = 'plaintext'

from Products.validation.interfaces import ivalidator

from Products.validation.config import validation
from Products.validation.interfaces.IValidator import IValidator

from Products.BrFieldsAndWidgets import MessageFactory as _

import re

listValidators = []

class ValidadorCPF:
    """
    Validador para verificar se o CPF informado e valido
    Baseado em http://www.pythonbrasil.com.br/moin.cgi/VerificadorDeCPF .
    """
    
    __implements__ = IValidator
    
    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description
    
    
    def __call__(self, value, *args, **kw):
        cpf = value
        cpf = ''.join([c for c in value if c.isdigit()])
        
        if len(cpf) != 11:
            return _(u"CPF precisa ter 11 dígitos.")
        elif len(cpf)==11:
            vtemp = [int(cpf[:1]) for i in list(cpf)] 
            cpf2 = [int(i) for i in list(cpf)]
            if cpf2 == vtemp:
                return _(u"CPF inválido.")
            
            tmp = cpf[:9] 
            ltmp = [int(i) for i in list(tmp)]               
            
            while len(ltmp) < 11:
                R = sum(map(lambda(i,v):(len(ltmp)+1-i)*v,enumerate(ltmp))) % 11
                
                if R > 1:
                    f = 11 - R
                else:
                    f = 0
                ltmp.append(f)
            
            if cpf2 != ltmp:
                return _(u"O dígito verificador do CPF não confere.")
        return True
    

listValidators.append(ValidadorCPF('isCPF', title=_(u'Validator de CPF'), description=''))

class ValidadorCNPJ:
    """
    Validador para verificar se o CNPJ informado e valido.
    Baseado em http://www.pythonbrasil.com.br/moin.cgi/VerificadorDeCnpj
    """
    
    __implements__ = IValidator
    
    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description
    
    
    def __call__(self, value, *args, **kw):
        cnpj = value
        cnpj = ''.join([c for c in value if c.isdigit()])
        if len(cnpj) != 14:
            return _(u"O CNPJ deve ter 14 dígitos.")
        elif len(cnpj) == 14:
            vtemp = [int(cnpj[:1]) for i in list(cnpj[:8])] 
            cnpj2 = [int(i) for i in list(cnpj[:8])]
            
            if cnpj2 == vtemp:
               return _(u"O CNPJ informado é inválido.")
            
            tmp = cnpj[:12]
            ltmp = [int(i) for i in list(tmp)]
            temp = [int(i) for i in list(cnpj)]
            prod = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
            
            while len(ltmp) < 14:
                R = sum([x*y for (x, y) in zip(ltmp, prod)])%11
                if R >= 2:
                    f = 11 - R
                else:
                    f = 0
                ltmp.append(f)
                prod.insert(0,6)
            if temp != ltmp:
                return _(u"O CNPJ informado é inválido.")
        return True
    

listValidators.append(ValidadorCNPJ('isCNPJ', title='', description=''))

class ValidadorCEP:
    """
    Validador para informar se o CEP informado is valido. Sao aceitos codigos 
    de enderecamento postal em duas formas:Oito digitos consecutivos ou cinco 
    digitos, hifen, tres digitos, ou seja, XXXXXXXX ou XXXXX-XXX, onde cada X 
    pode ser qualquer digito entre 0 e 9.
    """
    
    __implements__ = IValidator
    
    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description
    
    
    def __call__(self, value, *args, **kw):
        cep = ''.join([c for c in value if c.isdigit()])
        
        if not(len(cep)==8):
            return _(u"O cep informado é inválido.")
        return True

listValidators.append(ValidadorCEP('isCEP', title='CEP Validator', description=''))

class ValidadorBrPhone:
    """
    Validador para telefones brasileiros. Suportando os formatos:
        XXXXXXXXXX (1155553211) - Novos telefones
        XXXXXXXXX  (115552133) - Antigos telefones
        0n00XXXXXXX (n sendo 3 ou 8)
        0n00XXXXXX (n sendo 3 ou 8)
    """
    
    __implements__ = IValidator
    
    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description
    
    def __call__(self, value, *args, **kw):
        
        phone = ''.join([c for c in value if c.isdigit()])
        status = True
        
        if phone.startswith('0'):
            if not((len(phone) in [10,11]) and phone.isdigit()):
                status = False
        elif not(len(phone) in [9,10] and phone.isdigit()):
            status = False
        
        return status or _(u"Telefone inválido")
    

listValidators.append(ValidadorBrPhone('isBrPhone', title='Brazilian Phone Validator', description=''))

for validador in listValidators:
    validation.register(validador)



""" demonstrates the use of BrFieldsAndWidgets """

from Products.Archetypes.public import *

from Products.BrFieldsAndWidgets import MessageFactory as _

from AccessControl import ClassSecurityInfo

from Products.BrFieldsAndWidgets.content.BrFieldsAndWidgets import *
from Products.BrFieldsAndWidgets.validators import ValidadorCNPJ
from Products.BrFieldsAndWidgets.validators import ValidadorCPF
from Products.BrFieldsAndWidgets.validators import ValidadorCEP
from Products.BrFieldsAndWidgets.validators import ValidadorBrPhone

schema = BaseSchema +  Schema((
    CPFField('cpf',
                searchable=1,
                ),
    CEPField('cep',
                searchable=1,
                ),
    CNPJField('cnpj',
                searchable=1,
                ),
    BrPhoneField('phone',
                searchable=1,
                ),
))

class BrFieldsAndWidgetsDemo(BaseContent):
    """
    Demo for BrFieldsAndWidgets
    """
    meta_type = portal_type = archetype_name = 'BrFieldsAndWidgetsDemo'
    content_icon = "document_icon.gif"
    schema = schema
    security = ClassSecurityInfo()


registerType(BrFieldsAndWidgetsDemo)



from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import StringWidget
from Products.Archetypes.Registry import registerWidget
from Products.Archetypes.Field import StringField

from Products.BrFieldsAndWidgets import MessageFactory as _

class CPFWidget(StringWidget):
    security = ClassSecurityInfo()
    
    _properties = StringWidget._properties.copy()
    _properties.update({
        'macro': 'BrCPF',
        'helper_js': ('BrFieldsAndWidgets.js',),
        })
    


class CPFField(StringField):
    security  = ClassSecurityInfo()    
    _properties = StringField._properties.copy()
    _properties.update({
        'widget': CPFWidget,
        'validators': ('isCPF',),
        })
    


class CEPWidget(StringWidget):
    security = ClassSecurityInfo()
    
    _properties = StringWidget._properties.copy()
    _properties.update({
        'macro': 'BrCEP',
        'helper_js': ('BrFieldsAndWidgets.js',),
        })
    


class CEPField(StringField):
    security  = ClassSecurityInfo()    
    _properties = StringField._properties.copy()
    _properties.update({
        'widget': CEPWidget,
        'validators': ('isCEP',),
        })
    


class CNPJWidget(StringWidget):
    security = ClassSecurityInfo()
    
    _properties = StringWidget._properties.copy()
    _properties.update({
        'macro': 'BrCNPJ',
        'helper_js': ('BrFieldsAndWidgets.js',),
        })
    


class CNPJField(StringField):
    security  = ClassSecurityInfo()    
    _properties = StringField._properties.copy()
    _properties.update({
        'widget': CNPJWidget,
        'validators': ('isCNPJ',),
        })
    


class BrPhoneWidget(StringWidget):
    security = ClassSecurityInfo()
    
    _properties = StringWidget._properties.copy()
    _properties.update({
        'macro': 'BrPhone',
        'helper_js': ('BrFieldsAndWidgets.js',),
        })
    


class BrPhoneField(StringField):
    security  = ClassSecurityInfo()    
    _properties = StringField._properties.copy()
    _properties.update({
        'widget': BrPhoneWidget,
        'validators': ('isBrPhone',),
        })
    


class BrTimeWidget(StringWidget):
    security = ClassSecurityInfo()
    
    _properties = StringWidget._properties.copy()
    _properties.update({
        'macro': 'BrTime',
        'helper_js': ('BrFieldsAndWidgets.js',),
        })


registerWidget(CPFWidget,
               title='CPF',
               description="CPF widget.",
               used_for=('Products.Archetypes.public.StringField',)
               )

registerWidget(CEPWidget,
               title='CEP',
               description="CEP widget.",
               used_for=('Products.Archetypes.public.StringField',)
               )

registerWidget(CNPJWidget,
               title='CNPJ',
               description="CNPJ widget.",
               used_for=('Products.Archetypes.public.StringField',)
               )

registerWidget(BrPhoneWidget,
               title='BrPhone',
               description="BrPhone widget.",
               used_for=('Products.Archetypes.public.StringField',)
               )

registerWidget(BrTimeWidget,
               title='BrTimeWidget',
               description="BrTime widget.",
               used_for=('Products.Archetypes.public.StringField',)
               )


import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.BrFieldsAndWidgets.tests.tests import TestCase
from Products.Archetypes.public import *

class TestFields(TestCase):
    """ Unit test cases for BrFieldsAndWidgets fields definition """

    def afterSetUp(self):
        self.loginAsPortalOwner()
        self.portal.invokeFactory('BrFieldsAndWidgetsDemo', 'demo')
        self.demo = self.portal.demo


    def testCpfFill(self):
        """ Test that it is possible to enter data in a Cpf Field"""
        self.demo.setCpf('81057807559')
        self.field = self.demo.getField('cpf')
    
    def testCnpjFill(self):
        """ Test that it is possible to enter data in a CNPJ Field"""
        self.demo.setCnpj('90.656.353/0001-37')
        self.field = self.demo.getField('cnpj')

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestFields))
    return suite

if __name__ == '__main__':
    framework()



# -*- coding: utf-8 -*-
__author__ = """Simples Consultoria <contato@simplesconsultoria.com.br>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *

from Products.BrFieldsAndWidgets.content.BrFieldsAndWidgets import *
from Products.CMFPlone.utils import getToolByName

from Products.BrFieldsAndWidgets import MessageFactory as _

schema = Schema((
    StringField(
        name='logradouro',
        widget=StringWidget(
            size="30",
            description="Exemplo: Rua Mourato Coelho",
            visible={'view':'invisible', 'edit':'visible'},
            label='Logradouro',
            label_msgid='BrFieldsAndWidgets_label_logradouro',
            description_msgid='BrFieldsAndWidgets_help_logradouro',
            i18n_domain='Products.BrFieldsAndWidgets',
        ),
        required=1,
        schemata="Address"
    ),
    
    StringField(
        name='numero',
        widget=StringWidget(
            size="7",
            description="12",
            visible={'view':'invisible', 'edit':'visible'},
            label='Número',
            label_msgid='BrFieldsAndWidgets_label_numero',
            description_msgid='BrFieldsAndWidgets_help_numero',
            i18n_domain='Products.BrFieldsAndWidgets',
        ),
        required=1,
        schemata="Address"
    ),
    
    StringField(
        name='complemento',
        widget=StringWidget(
            visible={'view':'invisible', 'edit':'visible'},
            label='Complemento',
            label_msgid='BrFieldsAndWidgets_label_complemento',
            i18n_domain='Products.BrFieldsAndWidgets',
        ),
        schemata="Address"
    ),
    
    StringField(
        name='bairro',
        widget=StringWidget
        (
            description="Exemplo: Pinheiros.",
            visible={'view':'invisible', 'edit':'visible'},
            size="30",
            label='Bairro',
            label_msgid='BrFieldsAndWidgets_label_bairro',
            description_msgid='BrFieldsAndWidgets_help_bairro',
            i18n_domain='Products.BrFieldsAndWidgets',
        ),
        required=0,
        schemata="Address",
    ),
    
    CEPField(
        name='cep',
        required=1,
        widget=CEPWidget(   
                        description="Informe o CEP.",
                        visible={'view':'invisible', 'edit':'visible'},
                        label='CEP',
                        label_msgid='BrFieldsAndWidgets_label_cep',
                        description_msgid='BrFieldsAndWidgets_help_cep',
                        ),
        schemata="Address"
    ),
    
    StringField(
        name='cidade',
        widget=StringWidget(
            description="Informe sua cidade.",
            visible={'view':'invisible', 'edit':'visible'},
            label='Cidade',
            label_msgid='BrFieldsAndWidgets_label_cidade',
            description_msgid='BrFieldsAndWidgets_help_cidade',
            i18n_domain='Products.BrFieldsAndWidgets',
        ),
        required=1,
        schemata="Address"
    ),
    
    StringField(
        name='uf',
        widget=SelectionWidget
        (
            description="Informe seu estado.",
            visible={'view':'invisible', 'edit':'visible'},
            size="1",
            label='Uf',
            label_msgid='BrFieldsAndWidgets_label_uf',
            description_msgid='BrFieldsAndWidgets_help_uf',
            i18n_domain='Products.BrFieldsAndWidgets',
        ),
        required=1,
        schemata="Address",
        vocabulary='vocLstUF',
        enforceVocabulary=1
    ),
    
    ComputedField(
        name='Endereco',
        expression='context.fmt_endereco()',
        widget=ComputedWidget(
            visible={'view':'visible', 'edit':'invisible'},
            label='Endereco',
            label_msgid='BrFieldsAndWidgets_label_Endereco',
            i18n_domain='Products.BrFieldsAndWidgets',
        ),
        default_output_type='text/x-html-safe',
        schemata="Address",
        searchable=1
    ),
    
    TextField(
        name='referencia',
        allowable_content_types=('text/html',),
        widget=RichWidget
        (
            description="Informe pontos de referência.",
            visible={'view':'invisible', 'edit':'invisible'},
            label='Referencia',
            label_msgid='BrFieldsAndWidgets_label_referencia',
            description_msgid='BrFieldsAndWidgets_help_referencia',
            i18n_domain='Products.BrFieldsAndWidgets',
        ),
        required=0,
        schemata="Address",
        searchable=1,
        default_output_type='text/x-html-safe'
    ),

),
)

br_endereco_schema = BaseSchema.copy() + \
    schema.copy()


class br_endereco:
    """Classe basica de endereco
    """
    security = ClassSecurityInfo()

    # This name appears in the 'add' box
    archetype_name = 'br_endereco'

    meta_type = 'br_endereco'
    portal_type = 'br_endereco'
    allowed_content_types = []
    filter_content_types = 0
    global_allow = 0
    #content_icon = 'br_endereco.gif'
    immediate_view = 'base_view'
    default_view = 'base_view'
    suppl_views = ()
    typeDescription = "br_endereco"
    typeDescMsgId = 'description_edit_br_endereco'

    _at_rename_after_creation = True

    schema = br_endereco_schema

    security.declarePrivate('fmt_endereco')
    def fmt_endereco(self):
        """Retorna um endereco formatado para o atributo Endereco
        """
        template = "%s,%s %s<br/>%s<br/>%s<br/>%s - %s"
        logradouro = self.getLogradouro()
        numero = self.getNumero()
        complemento = self.getComplemento()
        bairro = self.getBairro()
        cep = self.getCep()
        if cep:
            cep = "%s-%s" % (cep[:5], cep[-3:])
        cidade = self.getCidade()
        uf= self.getUf()

        return template % (logradouro, numero, complemento, bairro, cep, cidade, uf)


    security.declarePrivate('vocLstUF')
    def vocLstUF(self):
        """Retorna lista de unidades da federacao
        """
        uf = [
            ('','Selecione'),
            ('AC',u'Acre'),
            ('AL',u'Alagoas'),
            ('AM',u'Amazonas'),
            ('AP',u'Amapá'),
            ('BA',u'Bahia'),
            ('CE',u'Ceará'),
            ('DF',u'Distrito Federal'),
            ('ES',u'Espírito Santo'),
            ('GO',u'Goiás'),
            ('MA',u'Maranhão'),
            ('MG',u'Minas Gerais'),
            ('MS',u'Mato Grosso do Sul'),
            ('MT',u'Mato Grosso'),
            ('PA',u'Pará'),
            ('PB',u'Paraíba'),
            ('PE',u'Pernambuco'),
            ('PI',u'Piauí'),
            ('PR',u'Paraná'),
            ('RJ',u'Rio de Janeiro'),
            ('RN',u'Rio Grande do Norte'),
            ('RO',u'Rondônia'),
            ('RR',u'Roraima'),
            ('RS',u'Rio Grande do Sul'),
            ('SC',u'Santa Catarina'),
            ('SE',u'Sergipe'),
            ('SP',u'São Paulo'),
            ('TO',u'Tocantins')]

        return uf
